﻿// Fill out your copyright notice in the Description page of Project Settings.


#include "AI_EnemyController.h"

#include "EnemyCharacter.h"
#include "BehaviorTree/BlackboardComponent.h"
#include "Perception/AIPerceptionComponent.h"
#include "Perception/AISenseConfig_Sight.h"


// Sets default values
AAI_EnemyController::AAI_EnemyController(FObjectInitializer const& ObjectInitializer)
{
	// Set this actor to call Tick() every frame.  You can turn this off to improve performance if you don't need it.
	PrimaryActorTick.bCanEverTick = true;

	SetupPerceptionSystem();
}

// Called when the game starts or when spawned
void AAI_EnemyController::BeginPlay()
{
	Super::BeginPlay();
}

void AAI_EnemyController::OnPossess(APawn* InPawn)
{
	Super::OnPossess(InPawn);
	if (const AEnemyCharacter* EnemyCharacter = Cast<AEnemyCharacter>(InPawn))
	{
		if (UBehaviorTree* const BehaviorTree = EnemyCharacter->GetBehaviorTree())
		{
			UBlackboardComponent* TempBlackboardPtr;
			UseBlackboard(BehaviorTree->BlackboardAsset, TempBlackboardPtr);
			Blackboard = TempBlackboardPtr;
			RunBehaviorTree(BehaviorTree);
		}
	}
}

// Called every frame
void AAI_EnemyController::Tick(float DeltaTime)
{
	Super::Tick(DeltaTime);
}

void AAI_EnemyController::SetupPerceptionSystem()
{
	SightConfig = CreateDefaultSubobject<UAISenseConfig_Sight>(TEXT("Sight Config"));
	if (IsValid(SightConfig))
	{
		SetPerceptionComponent(*CreateDefaultSubobject<UAIPerceptionComponent>(TEXT("Perception Component")));
		SightConfig->SightRadius = 1000.0f;
		SightConfig->LoseSightRadius = 1100.0f;
		SightConfig->PeripheralVisionAngleDegrees = 90.0f;
		SightConfig->SetMaxAge(5.0f);
		SightConfig->AutoSuccessRangeFromLastSeenLocation = 520.0f;
		SightConfig->DetectionByAffiliation.bDetectEnemies = true;
		SightConfig->DetectionByAffiliation.bDetectFriendlies = true;
		SightConfig->DetectionByAffiliation.bDetectNeutrals = true;
		GetPerceptionComponent()->SetDominantSense(*SightConfig->GetSenseImplementation());
		GetPerceptionComponent()->OnTargetPerceptionUpdated.AddDynamic(this, &AAI_EnemyController::OnTargetPerceptionUpdated);
		GetPerceptionComponent()->ConfigureSense(*SightConfig);
	}
}

void AAI_EnemyController::OnTargetPerceptionUpdated(AActor* Actor, FAIStimulus const Stimulus)
{
	if (AEnemyCharacter* const EnemyCharacter = Cast<AEnemyCharacter>(Actor))
	{
		if (Stimulus.WasSuccessfullySensed())
		{
			GetBlackboardComponent()->SetValueAsObject("TargetActor", Actor);
			GetBlackboardComponent()->SetValueAsVector("TargetLocation", Stimulus.StimulusLocation);
		}
		else
		{
			GetBlackboardComponent()->ClearValue("TargetActor");
			GetBlackboardComponent()->ClearValue("TargetLocation");
		}
	}
}
